/* Messages Macrocoder demo - library file - Copyright (c) 2016 by DAFOCUS */
#include "messagesLib.h"

#define POS(bit) (1 << (7-(bit & 7)))
#define SRC (*(((const unsigned char*) source)+(sourceOffsetInBits >> 3)))
#define DST (*(((unsigned char*)destination)+(destinationOffsetInBits >> 3)))

/*------------------------------------------------------------------------------
This function copies given number of bits starting from given sourceOffsetInBits
from "source" buffer to "destination" buffer.  Bits are aligned on destination
buffer starting from bit "destinationOffsetInBits".

Bit-positions are expressed as follows:
- bit 0 is the highest (most significant) bit of octet 0;
- bit 7 is the lowest (less significant) bit of octet 0;
- bit 8 is the highest (most significant) bit of octet 1;
...and so on.

Both Lenght and Offsets are expressed in bits.
------------------------------------------------------------------------------*/
void copyBits(void* destination, unsigned destinationOffsetInBits, const void* source, unsigned sourceOffsetInBits, unsigned lengthInBits)
{
	unsigned l;

	for (l = 0; l<lengthInBits; l++) {
		if ((SRC & POS(sourceOffsetInBits)) != 0) DST |= POS(destinationOffsetInBits);
		else DST &= ~POS(destinationOffsetInBits);
		sourceOffsetInBits++;
		destinationOffsetInBits++;
	}
}

/*------------------------------------------------------------------------------
This function copies 'lengthInBits' less significant bits of 'source' integer to
'destination' ad offset 'Offset'.
------------------------------------------------------------------------------*/
void unsignedToBits(void* destination, unsigned destinationOffsetInBits, unsigned long source, unsigned lengthInBits)
{
	unsigned i;
	for (i = lengthInBits - 1; i >= 0; i--) {
		if ((source & (1 << i)) != 0) DST |= POS(destinationOffsetInBits);
		else DST &= ~POS(destinationOffsetInBits);
		destinationOffsetInBits++;
	}
}

/*------------------------------------------------------------------------------
This function copies 'lengthInBits' less significant bits of 'source' char buffer
from offset 'Offset'. Read integer is returned.
------------------------------------------------------------------------------*/
unsigned long bitsToUnsigned(const void* source, unsigned sourceOffsetInBits, unsigned lengthInBits)
{
	unsigned i;
	unsigned long ret = 0;
	for (i = 0; i<lengthInBits; i++) {
		if ((SRC & POS(sourceOffsetInBits)) != 0) ret |= (1 << (lengthInBits - i - 1));
		sourceOffsetInBits++;
	}
	return ret;
}
